<?php
/*
 *Name         : helper.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : Dec 1, 2009
 *History      : 0.3  
 *Purpose      : Contains utility functions used by the library. 
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

/*###############################################
 * Sends a message to the server and listens for a reply
 * Takes: request
 * Returns: an error or the server reply
 */
function sendAndRecive($request) {
	
	global $SOCKET, $READ_LENGTH, $NEW_LINE;
	
	//Add new line to request
	$request = $request . $NEW_LINE;
		
	//Check if socket is initialized 
	if(($SOCKET !== null) && ($SOCKET !== false)) {
		
		//Read out prompt
		if(($buf = socket_read($SOCKET, $READ_LENGTH, PHP_BINARY_READ)) !== false) {
			
			//Send request
			if(socket_write($SOCKET, $request, strlen($request))) {
				
				//Read reply					
				if(($buf = socket_read($SOCKET, $READ_LENGTH, PHP_BINARY_READ)) !== false) {
					return $buf;
				} else {
					return "Error: Could not read from socket. " . socket_strerror(socket_last_error());
				}
			} else {
				return "Error: Could not write to socket. " . socket_strerror(socket_last_error());
			}
		} else {
			return "Error: Could not read from socket. " . socket_strerror(socket_last_error());
		}
	} else {
		return "Error: Socket not connected. Call iSymphonyConnect() to initialize connection.";
	}	
}
	
/*###############################################
 * Checks if reply was an error
 * Takes: server reply
 * Returns: true or false
 */	
function isError($reply) {
	return strpos($reply, "Error") === 0;
}	

/*###############################################
 * Makes a request that returns nothing and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: true if successful and false if error 
 */
function checkAndSetErrorNone($request) {
	
	global $ISERROR;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		return true;
	}
}

/*###############################################
 * Makes a request that returns a string and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: the string if successful and false if error 
 */
function checkAndSetErrorString($request) {
	
	global $ISERROR;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		return trim($reply);
	}
}

/*###############################################
 * Makes a request that returns a list and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: an array of values if successful and false if error 
 */
function checkAndSetErrorList($request) {
	
	global $ISERROR, $NEW_LINE;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		$explodeArray = explode($NEW_LINE, trim($reply));
		$returnArray = array();
	
		//Search array for empty values and remove them
		foreach($explodeArray as $val) {
			if(strlen($val) != 0) {
				array_push($returnArray, $val);
			}
		}
				
		return $returnArray;
	}
}

/*###############################################
 * Coverts a property list into a associative key => value array 
 * Takes: an array of properties with values structured like so (property = value)
 * Returns: an associative array with the structure (property => value)
 */
function convertPropertyArray($array) {
	
	$returnArray = array();
	
	foreach($array as $val) {
	
		//Check if the property has a value and set the key value pair appropriately 
		if(endsWith($val," =")) {
			$keyValuePair = array(rtrim($val," =") ,"");
		} else if(endsWith($val," = ")) {
			$keyValuePair = array(rtrim($val," = "),"");
		} else {
			$keyValuePair = explode(" = ", $val);
		}

		$returnArray[$keyValuePair[0]] = $keyValuePair[1];
	}
	
	return $returnArray;
}

/*###############################################
 * Checks if a string ends with another specified string
 * Takes: a string, the string you wish to check is at the end
 * Returns: true if $endString is at the end of $string false if not
 */
function endsWith($string, $endString){
    return strrpos($string, $endString) === strlen($string)-strlen($endString);
}

/*###############################################
 * Removes an element from an array by value
 * Takes: an array and the element to be removed
 * Returns: and new re-orderd array without the removed element
 */
function remove_element($arr, $val){
	foreach ($arr as $key => $value){
		if ($arr[$key] == $val){
			unset($arr[$key]);
		}
	}
	return $arr = array_values($arr);
}

/*###############################################
 * Sets the current state of the tracker variables
 * Takes: the new location, tenant, profile, and/or group
 * Returns: nothing
 */
function update_tracker($location = null, $tenant = null, $profile = null, $group = null) {
	global $CURRENT_LOCATION, $CURRENT_TENANT, $CURRENT_PROFILE, $CURRENT_GROUP;
	
	$CURRENT_LOCATION = $location;
	$CURRENT_TENANT = $tenant;
	$CURRENT_PROFILE = $profile;
	$CURRENT_GROUP = $group;
}

/*###############################################
 * Checks to see if CLI is in the given location
 * Takes: location
 * Returns: true if the CLI is currently in this location and false otherwise
 */
function in_location($location) {
	global $CURRENT_LOCATION;
	return ($CURRENT_LOCATION === $location);
}

/*###############################################
 * Checks to see if CLI is in the given tenant
 * Takes: location and tenant
 * Returns: true if the CLI is currently in this tenant and false otherwise
 */
function in_tenant($location, $tenant) {
	global $CURRENT_LOCATION, $CURRENT_TENANT;
	return (($CURRENT_LOCATION === $location) && 
			($CURRENT_TENANT === $tenant));
}

/*###############################################
 * Checks to see if CLI is in the given profile
 * Takes: location, tenant, and profile
 * Returns: true if the CLI is currently in this profile and false otherwise
 */
function in_profile($location, $tenant, $profile) {
	global $CURRENT_LOCATION, $CURRENT_TENANT, $CURRENT_PROFILE;
	return (($CURRENT_LOCATION === $location) && 
			($CURRENT_TENANT === $tenant) &&
			($CURRENT_PROFILE === $profile));
}

/*###############################################
 * Checks to see if CLI is in the given group
 * Takes: location, tenant, and group
 * Returns: true if the CLI is currently in this group and false otherwise
 */
function in_group($location, $tenant, $group) {
	global $CURRENT_LOCATION, $CURRENT_TENANT, $CURRENT_GROUP;
	return (($CURRENT_LOCATION === $location) && 
			($CURRENT_TENANT === $tenant) &&
			($CURRENT_GROUP === $group));
}

?>