<?php
/*
 *Name         : functions.inc.php
 *Author       : Michael Yara
 *Created      : June 27, 2008
 *Last Updated : Nov 23, 2010
 *History      : 2.1
 *Purpose      : FreePBX module that automatically updates the iSymphony configuration from the FreePBX configuration.
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

//Build include path for iSymphony library
$inc_path = __FILE__;
$pos = strrpos($inc_path, "/");
$inc_path = substr($inc_path, 0, $pos) . "/";

//Include iSymphony library
require_once($inc_path . "/isymphony-php-library/isymphony.php");

//ParkAndAnnounce Class
class ext_isymphony_parkandannounce {
	var $template;
	var $timeout;
	var $dial;
	var $return_context;
	
	function ext_isymphony_parkandannounce($template, $timeout, $dial, $return_context) {
		$this->template = $template;
		$this->timeout = $timeout;
		$this->dial = $dial;
		$this->return_context = $return_context;
	}
	
	function output() {
		return "ParkAndAnnounce(" . $this->template . "," . $this->timeout . "," . $this->dial . "," . $this->return_context . ")";
	}
}

//Control
class ext_isymphony_controlplayback {
	var $fileName;
	var $skipMinutes;
	var $ff;
	var $rew;
	var $stop;
	var $pause;
	var $restart;
		
	function ext_isymphony_controlplayback($fileName, $skipMinutes, $ff, $rew, $stop, $pause, $restart) {
		$this->fileName = $fileName;
		$this->skipMinutes = $skipMinutes;
		$this->ff = $ff;
		$this->rew = $rew;
		$this->stop = $stop;
		$this->pause = $pause;
		$this->restart = $restart;
	}
	
	function output() {
		return "ControlPlayback(" . $this->fileName . "," . $this->skipMinutes . "," . $this->ff . "," . $this->rew . "," . $this->stop . "," . $this->pause . "," . $this->restart . ")";
	}
}

//ChanSpy Class
class ext_isymphony_chanspy {
	var $prefix;
	var $options;
	
	function ext_isymphony_chanspy($prefix, $options) {
		$this->prefix = $prefix;
		$this->options = $options;
	}
	
	function output() {
		return "ChanSpy(" . $this->prefix . "," . $this->options . ")";
	}
}

//If check box GUI element does not exist add it
if(!class_exists("gui_checkbox")) {
	class gui_checkbox extends guiinput {
	       function gui_checkbox($elemname, $checked=false, $prompttext='', $helptext='', $value='on', $post_text = '', $jsonclick = '', $disable=false) {
	               $parent_class = get_parent_class($this);
	               parent::$parent_class($elemname, '', $prompttext, $helptext);
	
	               $itemchecked = $checked ? 'checked' : '';
	               $disable_state = $disable ? 'disabled="true"' : '';
	               $js_onclick_include = ($jsonclick != '') ? 'onclick="' . $jsonclick. '"' : '';
	               $tabindex = function_exists("guielement::gettabindex") ? "tabindex=" . guielement::gettabindex() : ""; 

	               $this->html_input = "<input type=\"checkbox\" name=\"$this->_elemname\" id=\"$this->_elemname\" $disable_state $tabindex value=\"$value\" $js_onclick_include $itemchecked/>$post_text\n";
	       }
	}
}

//Main contributing module function------------------------------------------------------------------------------------------------------------------
function isymphony_get_config($engine) {
	
	global $ext, $ISERROR, $amp_conf, $db;
  	
  	switch($engine) {
  		case 'asterisk':	
  	
			ob_start();	
						
			$runningTimeStart = microtime(true);			
						
			//Open file for error log
			$errorLogFile = fopen("/var/www/html/admin/modules/isymphony/error.txt", 'w');
			
			//Open debug log
			if(($debugLogFile = fopen("/var/www/html/admin/modules/isymphony/debug.txt", 'w')) === false) {
				isymphony_write_to_file($errorLogFile, "Could not open debug log for writing.\n");
			}
						
			//Check if a manager profile exists for iSymphony if not create it.
			$managerFound = false;
			if((function_exists("manager_list")) && (($managers = manager_list()) !== null)) {
				
				//Search for iSymphony manager
				foreach($managers as $manager) {
					if($manager['name'] == "isymphony" ) {
						$managerFound = true;
						break;
					}
				}
			}

			//If not found create a manager profile for isymphony
			if((function_exists("manager_add")) && (function_exists("manager_gen_conf")) && (!$managerFound)) {
				manager_add("isymphony", "ismanager*con", "0.0.0.0/0.0.0.0", "127.0.0.1/255.255.255.0", "all", "all");
				manager_gen_conf();
			}
		
			//Determine agent login context based on FreePBX Version
			$freepbxVersion = get_framework_version();
			$freepbxVersion = $freepbxVersion ? $freepbxVersion : getversion();
			$agentLoginContext = "from-internal";
			if(version_compare_freepbx($freepbxVersion, "2.6", ">=")) {
				$agentLoginContext = "from-queue";
			}
			
			//Grab server version and revision
			$serverVersion = "2.0";
			$serverRevision = "1104";
			if(iSymphonyConnect()) {
				if((($serverVersionCheck = getiSymphonyServerVersion()) !== false)) {
					$serverVersion = $serverVersionCheck;
				} else {
					isymphony_write_to_file($errorLogFile, "Could not get server version. $ISERROR \n");
				}
				
				if((($serverRevisonCheck = getiSymphonyServerRevision()) !== false)) {
					$serverRevision = $serverRevisonCheck;
				} else {
					isymphony_write_to_file($errorLogFile, "Could not get server revision. $ISERROR \n");
				}
				
				iSymphonyDisconnect();
			} else {
				isymphony_write_to_file($errorLogFile, "Could not get server version/revision. $ISERROR \n");
			}
		
			isymphony_write_to_file($debugLogFile, "Server Version:" . $serverVersion . " Server Revision:" . $serverRevision . "\n");
		
			//Check if server version is 2.0 or 2.1 and add appropriate contexts
			if($serverRevision >= "1174") {
				
				isymphony_write_to_file($debugLogFile, "Using 2.1 contexts\n");
				
				//Determine context prefix
				if($serverRevision >= "1903") {
					$contextPrefix = "c-x-operator-panel";
				} else {
					$contextPrefix = "isymphony";
				}
				
				//Determine variable prefix
				if($serverRevision >= "1912") {
					$variablePrefix = "XMLNamespace";
				} else {
					$variablePrefix = "iSymphony";
				}
				
				//Query parking timeout
				$parkingTimeout = 200;
				$sql = "SELECT keyword, data FROM parkinglot WHERE id = '1'";
				$results = $db->getAssoc($sql);
				if(!DB::IsError($results)) {
					$parkingTimeout = $results['parkingtime'];
				}
				
				isymphony_write_to_file($debugLogFile, "Using context prefix \"{$contextPrefix}\"\n");
				isymphony_write_to_file($debugLogFile, "Using variable prefix \"{$variablePrefix}\"\n");
				
				$id = $contextPrefix . "-hold";
			    $c = '432111';
			    $ext->add($id, $c, '', new ext_musiconhold("\${{$variablePrefix}MusicOnHoldClass}"));
			    $ext->add($id, $c, '', new ext_hangup());
			
				$id = $contextPrefix . "-voice-mail";
			    $c = '432112';
			    $ext->add($id, $c, '', new ext_vm("\${{$variablePrefix}VoiceMailBox}@\${{$variablePrefix}VoiceMailBoxContext},u"));
			    $ext->add($id, $c, '', new ext_hangup());
			    
			    $id = $contextPrefix . "-meetme";
			    $c = '432113';
			    $ext->add($id, $c, '', new ext_meetme("\${{$variablePrefix}MeetMeRoomNumber}", "\${{$variablePrefix}MeetMeRoomOptions}", ""));
			    $ext->add($id, $c, '', new ext_hangup());
			    			    
			    $id = $contextPrefix . "-park";
			    $c = '432114';
			    $ext->add($id, $c, '', new ext_isymphony_parkandannounce("pbx-transfer:PARKED", "$parkingTimeout", "432116@" . $contextPrefix . "-park-announce-answer", "\${{$variablePrefix}ParkContext},\${{$variablePrefix}ParkExtension},1"));
			    $ext->add($id, $c, '', new ext_hangup());
			    
			    $id = $contextPrefix . "-park-announce-answer";
			    $c = '432116';
			    $ext->add($id, $c, '', new ext_answer());
			    $ext->add($id, $c, '', new ext_hangup());
			    
			    $id = $contextPrefix . "-listen-to-voice-mail";
			    $c = '432115';
			    $ext->add($id, $c, '', new ext_isymphony_controlplayback("\${{$variablePrefix}VoiceMailPath}", "1000", "*", "#", "7", "8" , "9"));
			    $ext->add($id, $c, '', new ext_hangup());
			   
			   	$id = $contextPrefix . "-spy";
			    $c = '432117';
			    $ext->add($id, $c, '', new ext_isymphony_chanspy("\${{$variablePrefix}ChanSpyChannel}", "\${{$variablePrefix}ChanSpyOptions}"));
			    $ext->add($id, $c, '', new ext_hangup());
			    
			} else {
				isymphony_write_to_file($debugLogFile, "Using 2.0 contexts\n");
				
				$id = "musiconhold";
			    $c = '1000';
			    $ext->add($id, $c, '', new ext_answer(""));
			    $ext->add($id, $c, '', new ext_setvar("isHoldContext", "\${DB(iSymphonyChannelHoldContexts/\${CHANNEL})}"));
			    $ext->add($id, $c, '', new ext_musiconhold("\${isHoldContext}"));
			    $ext->add($id, $c, '', new ext_dbdel("iSymphonyChannelHoldContexts/\${CHANNEL}"));
			    
			    $id = "xfer-2-vm";
			    $c = "_.";
			    $ext->add($id, $c, '', new ext_goto("1", "\${VM_PREFIX}\${EXTEN}", "from-internal"));
			    $ext->add($id, $c, '', new ext_hangup());
	
			    $id = "ibarge";
				$ext->add($id, "_1X.", '', new ext_meetme("\${EXTEN:1}", "qd", "271721"));
			    $ext->add($id, "_1X.", '', new ext_hangup());
			    $ext->add($id, "_2X.", '', new ext_meetme("\${EXTEN:1}", "aq", "271721"));
			    $ext->add($id, "_2X.", '', new ext_hangup());
	
				$id = "imeetme";
				$c = "_.";
				$ext->add($id, "h", '', new ext_hangup());
				$ext->add($id, $c, '', new ext_setvar("roomOptions", "\${DB(iSymphonyMeetMeOptions/\${EXTEN})}"));
				$ext->add($id, $c, '', new ext_meetme("\${EXTEN}", "\${roomOptions}", "271721"));
				$ext->add($id, $c, '', new ext_hangup());
	
				$id = "ipark";
				$c = "_.";
				$ext->add($id, "h", '', new ext_hangup());
				$ext->add($id, $c, '', new ext_setvar("parkContext", "\${DB(iSymphonyParkContexts/\${EXTEN})}"));
				$ext->add($id, $c, '', new ext_answer(""));
				$ext->add($id, $c, '', new ext_wait("1"));
				$ext->add($id, $c, '', new ext_isymphony_parkandannounce("", "", "", "\${parkContext},\${EXTEN},1"));
				$ext->add($id, $c, '', new ext_dbdel("iSymphonyParkContexts/\${EXTEN}"));
			}

			//Check the Asterisk version, USEDEVSTATE, and USEQUEUESTATE info to determine how the agent login iterface should be set.
			$agentInterfaceType = "none";
			$info = engine_getinfo();
			$devStateEnabled = isset($amp_conf["USEDEVSTATE"]) && isset($amp_conf["USEQUEUESTATE"]) && $amp_conf["USEDEVSTATE"] === true && $amp_conf["USEQUEUESTATE"] === true;
			
			if(version_compare($info["version"], "1.6", ">=") || (version_compare($info["version"], "1.4.25", ">=") && !$devStateEnabled)) {
				isymphony_write_to_file($debugLogFile, "Using peer agent interface\n");
				$agentInterfaceType = "peer";
			} else if(version_compare($info["version"], "1.4.25", ">=") && $devStateEnabled) {
				isymphony_write_to_file($debugLogFile, "Using hint agent interface\n");
				$agentInterfaceType = "hint";
			} else {
				isymphony_write_to_file($debugLogFile, "Using no agent interface\n");
				$agentInterfaceType = "none";
			}

			//Connect to iSymphony server to query and submit information
			if(iSymphonyConnect()) {
			
				//Grab location Information
				if(($locationInformation = isymphony_location_get()) !== null) {
					
					//Check if server properties need to be updated.
					if(($object = getISymphonyServer()) !== false) {
						
						if(($object->username != $locationInformation["admin_user_name"]) || (($object->password != $locationInformation["admin_password"]))) {
							$object->username = $locationInformation["admin_user_name"];
							$object->password = $locationInformation["admin_password"];
							
							if($object->update()) {
								isymphony_write_to_file($debugLogFile, "Updated server\n");
							} else {
								isymphony_write_to_file($debugLogFile, "(Update server) . $ISERROR . \n");
							}
						}
						
					} else {
						isymphony_write_to_file($errorLogFile, "(Query server)" . $ISERROR. "\n");
					}
					
					//Check if default location exists if not create it else update manager connection values to stay consistent
					if(($iSymphonyLocations = getISymphonyLocationList()) !== false) {
						
						$maskedAutoReloadFlag = ($locationInformation["auto_reload"] == 1) ? "true" : "false";	
						$maskedLogJabberMessagesFlag = ($locationInformation["log_jabber_messages"] == 1) ? "true" : "false";	
						
						if(!in_array("default", $iSymphonyLocations)) {
							$object = new ISymphonyLocation;
							$object->name = "default";
							$object->admin_password = "secret";
							$object->asterisk_host = "localhost";
							$object->asterisk_port = "5038";
							$object->asterisk_login = "isymphony";
							$object->asterisk_password = "ismanager*con";
							$object->originate_timeout = $locationInformation["originate_timeout"];
							$object->jabber_host = $locationInformation["jabber_host"];
							$object->jabber_domain = $locationInformation["jabber_domain"];
							$object->jabber_resource = $locationInformation["jabber_resource"];
							$object->jabber_port = $locationInformation["jabber_port"];
							$object->mask_jabber_user_name_with_profile = "false";
							$object->log_jabber_messages = $maskedLogJabberMessagesFlag;
							$object->device_user_mode = "true";
							$object->reload_on_dial_plan_reload = $maskedAutoReloadFlag;
							$object->force_client_update = "true";
							$object->voice_mail_directory = "/var/spool/asterisk/voicemail";
							
							if($object->add()) {
								isymphony_write_to_file($debugLogFile, "Added location default\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add location)" . $ISERROR . "\n");
							}
						} else {
							if(($object = getISymphonyLocation("default")) !== false) {
								
								//Check if update needs to occur		
								if(isymphony_location_update_check($serverRevision, $locationInformation, $maskedAutoReloadFlag, $maskedLogJabberMessagesFlag, $object)) {	
									
									$object->asterisk_host = "localhost";
									$object->asterisk_port = "5038";
									$object->asterisk_login = "isymphony";
									$object->asterisk_password = "ismanager*con";
									$object->originate_timeout = $locationInformation["originate_timeout"];
									$object->jabber_host = $locationInformation["jabber_host"];
									$object->jabber_domain = $locationInformation["jabber_domain"];
									$object->jabber_resource = $locationInformation["jabber_resource"];
									$object->jabber_port = $locationInformation["jabber_port"];
									$object->log_jabber_messages = $maskedLogJabberMessagesFlag;
									$object->device_user_mode = "true";
									$object->reload_on_dial_plan_reload = $maskedAutoReloadFlag;
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated location default\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update location)" . $ISERROR. "\n");	
									}
								}	
							} else {
								isymphony_write_to_file($errorLogFile, "(Query location)" . $ISERROR. "\n");
							}
						}
					} else {
						isymphony_write_to_file($errorLogFile, "(List locations)" . $ISERROR. "\n");
					}
		
					//Check if default tenant exists if not create it else update context values to stay consistent
					if(($iSymphonyTenants = getISymphonyTenantList("default")) !== false) {
						
						//Mask auto answer flag
						$maskedPagedStatusEnabledFlag = ($locationInformation["page_status_enabled"] == 1) ? "true" : "false";	
						$recordFileNameMask = ($serverRevision >= 1520) ? "%EXT%-%DATE_TIME_FORMAT[yyyyMMdd-HHmmss]%-%ID%-%LINKED_ID%" : "%EXT%-%NAME%-%CID_NAME%-%CID_NUMBER%-%DATE%-%TIME%";
						
						if(!in_array("default", $iSymphonyTenants)) {
							$object = new ISymphonyTenant;
							$object->name = "default";
							$object->admin_password = "secret";
							$object->originating_context = "from-internal";
							$object->redirecting_context = "from-internal";
							$object->agent_login_context = $agentLoginContext;
							$object->music_on_hold_class = "default";
							$object->page_status_enabled = $maskedPagedStatusEnabledFlag;
							$object->page_context = "ext-paging";
							$object->outside_line_number = "";
							$object->record_file_name = $recordFileNameMask;
							$object->record_file_extension = "wav";
							$object->mix_mode = "true";
							
							if($object->add("default")) {
								isymphony_write_to_file($debugLogFile, "Added tenant default\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add tenant)" . $ISERROR. "\n");
							}
						} else {
							if(($object = getISymphonyTenant("default", "default")) !== false) {
								
								//Check if update needs to occur
								if(isymphony_tenant_update_check($serverRevision, $locationInformation, $maskedPagedStatusEnabledFlag, $agentLoginContext, $object)) {
									
									$object->originating_context = "from-internal";
									$object->redirecting_context = "from-internal";
									$object->agent_login_context = $agentLoginContext;
									$object->page_status_enabled = $maskedPagedStatusEnabledFlag;
									$object->page_context = "ext-paging";
									$object->record_file_name = $recordFileNameMask;
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated tenant default\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update tenant)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query tenant)" . $ISERROR. "\n");
							}
						}
					} else {
						isymphony_write_to_file($errorLogFile, "(List tenants)" . $ISERROR. "\n");
					}
				} else {
					isymphony_write_to_file($errorLogFile, "Failed to grab location information". "\n");
				}
	
				//Grab list of configured extensions from the database	
				$freePBXTempUsers = isymphony_user_list();
				
				//Filter list to exclude extensions that are not marked for addition
				$freePBXUsers = array();
				foreach($freePBXTempUsers as $freePBXTempUser) {
					if($freePBXTempUser["add_extension"] == "1") {
						array_push($freePBXUsers, $freePBXTempUser);
					}
				}
				
				//Filter the previous list for profiles excluding ones not marked for profile addition
				$freePBXProfiles = array();
				foreach($freePBXUsers as $freePBXUser) {
					if($freePBXUser["add_profile"] == "1") {
						array_push($freePBXProfiles, $freePBXUser);
					}
				}
	
				//Add, edit and remove extensions and profiles
				if((($iSymphonyExtensions = getISymphonyExtensionList("default", "default")) !== false) && (($iSymphonyProfiles = getISymphonyProfileList("default", "default")) !== false)) {
					
					//Build array of freePBX extensions to compare to iSymphony list for deletes 
					$freePBXExtensionCheckDeleteArray = array();
					foreach($freePBXUsers as $freePBXUser) {
						array_push($freePBXExtensionCheckDeleteArray, $freePBXUser["user_id"]);
					}
					
					//Delete appropriate extensions
					$deleteExtensionArray = array_diff($iSymphonyExtensions, $freePBXExtensionCheckDeleteArray);
					foreach($deleteExtensionArray as $val) {
						if(removeISymphonyExtension("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted extension {$val}\n");
							$iSymphonyExtensions = isymphony_remove_array_item($iSymphonyExtensions, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete extension)" . $ISERROR. "\n");
						}
					}
									
					//Add and update extensions
					foreach($freePBXUsers as $freePBXUser) {

						//Mask auto answer flag
						$maskedAutoAnswerFlag = ($freePBXUser["auto_answer"] == 1) ? "true" : "false";
						
						//Build agent login interface for extension
						$agentLoginInterface = "";
						switch($agentInterfaceType) {
							case "peer":
								$agentLoginInterface = $freePBXUser["peer"];
								break;
							case "hint":
								$agentLoginInterface = ($serverRevision >= "2223") ? ("Hint:" . $freePBXUser["user_id"] . "@ext-local") : ($freePBXUser["user_id"] . "@ext-local");
								break;
							case "none";	
								$agentLoginInterface = "";
								break;	
						}
						
						//If extension does not exist add it else update it
						if(!in_array($freePBXUser["user_id"], $iSymphonyExtensions)) {
							
							//Add extension
							$object = new ISymphonyExtension;
							$object->extension_val = $freePBXUser["user_id"];
							$object->name = $freePBXUser["display_name"];
							$object->peer = $freePBXUser["peer"];
							$object->alt_origination_method = "";
							$object->voice_mail = $freePBXUser["user_id"];
							$object->cell_phone = $freePBXUser["cell_phone"];
							$object->email = $freePBXUser["email"];
							$object->auto_answer = $maskedAutoAnswerFlag;
							$object->agent_login_interface = $agentLoginInterface;
							$object->agent_login_penalty = "0";
							$object->agent_login_name = $freePBXUser["display_name"];
							
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added extension {$freePBXUser['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add extension)" . $ISERROR . "\n");
							}
						} else {
							
							//Query extension configuration and update values
							if(($object = getISymphonyExtension("default", "default", $freePBXUser["user_id"])) !== false) {		
													
								//Check if update needs to occur
								if(isymphony_extension_update_check($serverRevision, $freePBXUser, $maskedAutoAnswerFlag, $agentLoginInterface, $object)) {		
											
									$object->name = $freePBXUser["display_name"];
									$object->voice_mail = $freePBXUser["user_id"];
									$object->peer = $freePBXUser["peer"];
									$object->cell_phone = $freePBXUser["cell_phone"];
									$object->email = $freePBXUser["email"];
									$object->auto_answer = $maskedAutoAnswerFlag;
									$object->agent_login_interface = $agentLoginInterface;
									$object->agent_login_name = $freePBXUser["display_name"];
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated extension {$freePBXUser['user_id']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update extension)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query extension)" . $ISERROR. "\n"); 
							}
						}
					}
						
					//Build array of freePBX extensions to compare to iSymphony list for deletes 
					$freePBXProfileCheckDeleteArray = array();
					foreach($freePBXProfiles as $freePBXProfile) {
						array_push($freePBXProfileCheckDeleteArray, $freePBXProfile["user_id"]);
					}
						
					//Delete appropriate profiles
					$deleteProfileArray = array_diff($iSymphonyProfiles, $freePBXProfileCheckDeleteArray);
					foreach($deleteProfileArray as $val) {
						if(removeISymphonyProfile("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted profile {$val}\n");
							$iSymphonyProfiles = isymphony_remove_array_item($iSymphonyProfiles, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete profile)" . $ISERROR. "\n");
						}
					}	
						
					//Add and update profiles
					foreach($freePBXProfiles as $freePBXProfile) {
						
						//If profile does not exist add it else update it
						if(!in_array($freePBXProfile["user_id"], $iSymphonyProfiles)) {
							
							//Add profile
							$object = new ISymphonyProfile;
							$object->name = $freePBXProfile["user_id"];
							$object->password = $freePBXProfile["password"];
							$object->can_view_everyone_directory = "true";
							$object->jabber_host = $freePBXProfile["jabber_host"];
							$object->jabber_domain = $freePBXProfile["jabber_domain"];     
							$object->jabber_resource = $freePBXProfile["jabber_resource"];        
							$object->jabber_port = $freePBXProfile["jabber_port"]; 
							$object->jabber_user_name = $freePBXProfile["jabber_user_name"];
							$object->jabber_password = $freePBXProfile["jabber_password"];	
							
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added profile {$freePBXProfile['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add profile)" . $ISERROR . "\n");
							}
						
							//Add extension to profiles managed list
							if(addISymphonyProfileManagedExtension("default","default",$freePBXProfile["user_id"],$freePBXProfile["user_id"])) {
								isymphony_write_to_file($debugLogFile, "Added profile managed extension {$freePBXProfile['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add profile managed extension)" . $ISERROR . "\n");
							}
						} else {
							
							//Query profile configuration and update values
							if(($object = getISymphonyProfile("default", "default", $freePBXProfile["user_id"])) !== false) {		
													          
								//If jabber port is set to blank in the db rewrite with a -1 to prevent this profile from always being updated. Since the server will store a -1 as the port value if none is set.					
                        		$maskedJabberPort = ($object->jabber_port == -1) ? "" : $object->jabber_port; 
													                 
                                //Check if update needs to occur
                                if(isymphony_profile_update_check($serverRevision, $freePBXProfile, $maskedJabberPort, $object)) {                  
										
									$object->password = $freePBXProfile["password"];
									$object->jabber_host = $freePBXProfile["jabber_host"];
									$object->jabber_domain = $freePBXProfile["jabber_domain"];     
									$object->jabber_resource = $freePBXProfile["jabber_resource"];        
									$object->jabber_port = $freePBXProfile["jabber_port"]; 
									$object->jabber_user_name = $freePBXProfile["jabber_user_name"];
									$object->jabber_password = $freePBXProfile["jabber_password"];	
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated profile {$freePBXProfile['user_id']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update profile)" . $ISERROR. "\n");	
									}	
								}
								
								//Attempt to add relative extension to profile
								if(addISymphonyProfileManagedExtension("default","default",$freePBXProfile["user_id"],$freePBXProfile["user_id"])) {
									isymphony_write_to_file($debugLogFile, "Added profile managed extension {$freePBXProfile['user_id']}\n");
								}

							} else {
								isymphony_write_to_file($errorLogFile, "(Query profile)" . $ISERROR. "\n"); 
							}
						}												
					}
				} else {
					isymphony_write_to_file($errorLogFile, "(List extensions/profiles)" . $ISERROR. "\n");
				}	
				
				//Grab list of configured queues from the database	
				$freePBXTempQueues = isymphony_queue_list();
	
				//Filter list to exclude queues that are not marked for addition
				$freePBXQueues = array();
				foreach($freePBXTempQueues as $freePBXTempQueue) {
					if($freePBXTempQueue["add_queue"] == "1") {
						array_push($freePBXQueues, $freePBXTempQueue);
					}
				}
	
				//Add, edit and remove queues
				if(($iSymphonyQueues = getISymphonyQueueList("default", "default")) !== false) {
					
					//Build array of freePBX queues to compare to iSymphony list for deletes 
					$freePBXQueueCheckDeleteArray = array();
					foreach($freePBXQueues as $freePBXQueue) {
						array_push($freePBXQueueCheckDeleteArray, $freePBXQueue["display_name"]);
					}
				 	
				 	//Delete appropriate queues
					$deleteQueueArray = array_diff($iSymphonyQueues, $freePBXQueueCheckDeleteArray);
					foreach($deleteQueueArray as $val) {
						if(removeISymphonyQueue("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted queue {$val}\n");
							$iSymphonyQueues = isymphony_remove_array_item($iSymphonyQueues, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete queue)" . $ISERROR. "\n");
						}
					}
					
					//Add and update queues
				 	foreach($freePBXQueues as $freePBXQueue) {
				 					 		
				 		//If queue does not exist add it
						if(!in_array($freePBXQueue["display_name"], $iSymphonyQueues)) {
							
							//Add queue
							$object = new ISymphonyQueue;
							$object->name = $freePBXQueue["display_name"];
							$object->queue_val = $freePBXQueue["queue_id"];
							$object->extension_val = $freePBXQueue["queue_id"];
							$object->context = "from-internal";
	
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added queue {$freePBXQueue['display_name']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add queue)" . $ISERROR . "\n");
							}
							
						//If queue does exist update it		
						} else {
							
							//Query queue configuration and update values
							if(($object = getISymphonyQueue("default", "default", $freePBXQueue["display_name"])) !== false) {		
													
								//Check if update needs to occur
								if(($object->name != $freePBXQueue["display_name"]) || ($object->queue_val != $freePBXQueue["queue_id"]) || ($object->extension_val != $freePBXQueue["queue_id"]) || ($object->context != "from-internal")) {					
													
									$object->name = $freePBXQueue["display_name"];
									$object->queue_val = $freePBXQueue["queue_id"];
									$object->extension_val = $freePBXQueue["queue_id"];
									$object->context = "from-internal";
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated queue {$freePBXQueue['display_name']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update queue)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query queue)" . $ISERROR. "\n"); 
							}
						}	
				 	}				
				 } else {
				 	isymphony_write_to_file($errorLogFile, "(List queues)" . $ISERROR. "\n");
				 }
	
				//Grab list of configured conference rooms from the database	
				$freePBXTempConferenceRooms = isymphony_conference_room_list();
	
				//Filter list to exclude conference rooms that are not marked for addition
				$freePBXConferenceRooms = array();
				foreach($freePBXTempConferenceRooms as $freePBXTempConferenceRoom) {
					if($freePBXTempConferenceRoom["add_conference_room"] == "1") {
						array_push($freePBXConferenceRooms, $freePBXTempConferenceRoom);
					}
				}
	
				//Add, edit and remove conference rooms
				if(($iSymphonyConferenceRooms = getISymphonyConferenceRoomList("default", "default")) !== false) {
					
					//Build array of freePBX conference rooms to compare to iSymphony list for deletes 
					$freePBXConferenceRoomCheckDeleteArray = array();
					foreach($freePBXConferenceRooms as $freePBXConferenceRoom) {
						array_push($freePBXConferenceRoomCheckDeleteArray, $freePBXConferenceRoom["display_name"]);
					}
				 	
				 	//Delete appropriate conference rooms
					$deleteConferenceRoomArray = array_diff($iSymphonyConferenceRooms, $freePBXConferenceRoomCheckDeleteArray);
					foreach($deleteConferenceRoomArray as $val) {
						if(removeISymphonyConferenceRoom("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted conference room {$val}\n");
							$iSymphonyConferenceRooms = isymphony_remove_array_item($iSymphonyConferenceRooms, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete conference room)" . $ISERROR. "\n");
						}
					}
					
					//Add and update conference rooms
				 	foreach($freePBXConferenceRooms as $freePBXConferenceRoom) {
				 					 		
				 		//If conference room does not exist add it
						if(!in_array($freePBXConferenceRoom["display_name"], $iSymphonyConferenceRooms)) {
							
							//Add conference room
							$object = new ISymphonyConferenceRoom;
							$object->name = $freePBXConferenceRoom["display_name"];
							$object->predefined = "true";
							$object->room_number = $freePBXConferenceRoom["conference_room_id"];
							$object->extension_val = $freePBXConferenceRoom["conference_room_id"];
							$object->context = "from-internal";
							$object->announce_user_count = "false";
							$object->music_on_hold_for_single_user = "false";
							$object->exit_room_via_pound = "false";
							$object->present_menu_via_star = "false";
							$object->announce_user_join_leave = "false";
							$object->disable_join_leave_notification = "false";
							$object->record = "false";
	
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added conference room {$freePBXConferenceRoom['display_name']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add conference room)" . $ISERROR . "\n");
							}
							
						//If conference room does exist update it		
						} else {
							
							//Query conference room configuration and update values
							if(($object = getISymphonyConferenceRoom("default", "default", $freePBXConferenceRoom["display_name"])) !== false) {		
												
								//Check if update needs to occur
								if(($object->name != $freePBXConferenceRoom["display_name"]) || ($object->predefined != "true") || ($object->room_number != $freePBXConferenceRoom["conference_room_id"]) || ($object->extension_val != $freePBXConferenceRoom["conference_room_id"]) || ($object->context != "from-internal")) {
										
									$object->name = $freePBXConferenceRoom["display_name"];
									$object->predefined = "true";
									$object->room_number = $freePBXConferenceRoom["conference_room_id"];
									$object->extension_val = $freePBXConferenceRoom["conference_room_id"];
									$object->context = "from-internal";
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated conference room {$freePBXConferenceRoom['display_name']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update conference room)" . $ISERROR. "\n");	
									}
								}	
							} else {
								isymphony_write_to_file($errorLogFile, "(Query conference room)" . $ISERROR. "\n"); 
							}
						}	
				 	}				
				} else {
					isymphony_write_to_file($errorLogFile, "(List conference rooms)" . $ISERROR. "\n");
				}	
	
				//Close iSymphony connection
				iSymphonyDisconnect();
			
  			} else {
  				isymphony_write_to_file($errorLogFile, $ISERROR. "\n");
  			} 
			
			$runningTimeStop = microtime(true);
			isymphony_write_to_file($debugLogFile, "Total Running Time:" . ($runningTimeStop - $runningTimeStart) . "s\n");
			
			//Close file handlers
			fclose($debugLogFile);
			fclose($errorLogFile);
		
			ob_end_clean();
			break;
  	}
}

//iSymphony module API location functions------------------------------------------------------------------------------------------------------------
function isymphony_location_update($adminUserName, $adminPassword, $originateTimeout, $autoReload, $pageStatusEnabled, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $logJabberMessages) {
	global $db;	
	$autoReload = $autoReload ? "1" : "0";
	$pageStatusEnabled = $pageStatusEnabled ? "1" : "0";
	$prepStatement = $db->prepare("UPDATE isymphony_location SET admin_user_name = ?, admin_password = ?, originate_timeout = ?, auto_reload = ?, page_status_enabled = ?, jabber_host = ?, jabber_domain = ?, jabber_resource = ?, jabber_port = ?, log_jabber_messages = ?");
	$values = array($adminUserName, $adminPassword, $originateTimeout, $autoReload, $pageStatusEnabled, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $logJabberMessages);
	$db->execute($prepStatement, $values);
}

function isymphony_location_get() {
	global $db;	
	$query = "SELECT * FROM isymphony_location";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//iSymphony module API user functions----------------------------------------------------------------------------------------------------------------
function isymphony_user_add($userId, $addExtension, $addProfile, $password, $displayName, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword) {
	global $db;	
	$addProfile = $addProfile ? "1" : "0";
	$addExtension = $addExtension ? "1" : "0";
	$autoAnswer = $autoAnswer ? "1" : "0";
	$prepStatement = $db->prepare("INSERT INTO isymphony_users (user_id, add_extension, add_profile, password, display_name, peer, cell_phone, email, auto_answer, jabber_host, jabber_domain, jabber_resource, jabber_port, jabber_user_name, jabber_password) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ,?, ?, ?)");
	$values = array($userId, $addExtension, $addProfile, $password, $displayName, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword);
	$db->execute($prepStatement, $values);
}

function isymphony_user_update($userId, $addExtension, $addProfile, $password, $displayName, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword) {
	global $db;	
	$addProfile = $addProfile ? "1" : "0";
	$addExtension = $addExtension ? "1" : "0";
	$autoAnswer = $autoAnswer ? "1" : "0";
	$prepStatement = $db->prepare("UPDATE isymphony_users SET add_extension = ?, add_profile = ?, password = ?, display_name = ?, peer = ?, cell_phone = ?, email = ?, auto_answer = ?, jabber_host = ?, jabber_domain = ?, jabber_resource = ?, jabber_port = ?, jabber_user_name = ?, jabber_password = ? WHERE user_id = $userId");
	$values = array($addExtension, $addProfile, $password, $displayName, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword);
	$db->execute($prepStatement, $values);
}

function isymphony_user_del($userId) {
	global $db;	
	$query = "DELETE FROM isymphony_users WHERE user_id = '$userId'";
    $db->query($query);
}

function isymphony_user_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_users";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_user_get($userId) {
	global $db;	
	$query = "SELECT * FROM isymphony_users WHERE user_id = '$userId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//iSymphony module API queue functions---------------------------------------------------------------------------------------------------------------
function isymphony_queue_add($queueId, $addQueue, $displayName) {
	global $db;
	$addQueue = $addQueue ? "1" : "0";	
	$prepStatement = $db->prepare("INSERT INTO isymphony_queues (queue_id, add_queue, display_name) VALUES (?, ?, ?)");
	$values = array($queueId, $addQueue, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_queue_update($queueId, $addQueue, $displayName) {
	global $db;	
	$addQueue = $addQueue ? "1" : "0";	
	$prepStatement = $db->prepare("UPDATE isymphony_queues SET add_queue = ?, display_name = ? WHERE queue_id = $queueId");
	$values = array($addQueue, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_queue_del($queueId) {
	global $db;	
	$query = "DELETE FROM isymphony_queues WHERE queue_id = '$queueId'";
    $db->query($query);
}

function isymphony_queue_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_queues";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_queue_get($queueId) {
	global $db;	
	$query = "SELECT * FROM isymphony_queues WHERE queue_id = '$queueId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//iSymphony module API conference room functions-----------------------------------------------------------------------------------------------------
function isymphony_conference_room_add($conferenceRoomId, $addConferenceRoom, $displayName) {
	global $db;	
	$addConferenceRoom = $addConferenceRoom ? "1" : "0";	
	$prepStatement = $db->prepare("INSERT INTO isymphony_conference_rooms (conference_room_id, add_conference_room, display_name) VALUES (?, ?, ?)");
	$values = array($conferenceRoomId, $addConferenceRoom, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_conference_room_update($conferenceRoomId, $addConferenceRoom, $displayName) {
	global $db;	
	$addConferenceRoom = $addConferenceRoom ? "1" : "0";
	$prepStatement = $db->prepare("UPDATE isymphony_conference_rooms SET add_conference_room = ?, display_name = ? WHERE conference_room_id = $conferenceRoomId");
	$values = array($addConferenceRoom, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_conference_room_del($conferenceRoomId) {
	global $db;	
	$query = "DELETE FROM isymphony_conference_rooms WHERE conference_room_id = '$conferenceRoomId'";
    $db->query($query);
}

function isymphony_conference_room_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_conference_rooms";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_conference_room_get($conferenceRoomId) {
	global $db;	
	$query = "SELECT * FROM isymphony_conference_rooms WHERE conference_room_id = '$conferenceRoomId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//Extension/User page hooks--------------------------------------------------------------------------------------------------------------------------
function isymphony_configpageinit($pagename) {
	global $currentcomponent;

	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$extdisplay = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	$extension = isset($_REQUEST["extension"]) ? $_REQUEST["extension"] : null;
	$tech_hardware = isset($_REQUEST["tech_hardware"]) ? $_REQUEST["tech_hardware"] : null;

	//Based on the page state determine if the display or process functions should be added
	if (($pagename != "users") && ($pagename != "extensions")) { 
		return;
	} else if ($tech_hardware != null || $pagename == "users") {
		isymphony_extension_applyhooks();
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	} elseif ($action == "add" || $action == "edit") {
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	} elseif ($extdisplay != '') {
		isymphony_extension_applyhooks();
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	}
}

function isymphony_extension_applyhooks() {
	global $currentcomponent;
	$currentcomponent->addguifunc("isymphony_extension_configpageload");
}

function isymphony_extension_configpageload() {
	global $currentcomponent;

	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = isset($_REQUEST["display"]) ? $_REQUEST["display"] : null;
	$extension = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	
	//Attempt to query element if not found set defaults
	if(($extension !== null) && (($iSymphonyUser = isymphony_user_get($extension)) !== null)) {
		$added = ($iSymphonyUser["add_extension"] == "1");
		$userProfile = ($iSymphonyUser["add_profile"] == "1");
		$password = $iSymphonyUser["password"];
		$email	= $iSymphonyUser["email"];
		$cell_phone = $iSymphonyUser["cell_phone"];
		$autoAnswer = $iSymphonyUser["auto_answer"];
		$jabberHost = $iSymphonyUser["jabber_host"];
		$jabberDomain = $iSymphonyUser["jabber_domain"];
		$jabberResource = $iSymphonyUser["jabber_resource"];
		$jabberPort = $iSymphonyUser["jabber_port"];
		$jabberUserName = $iSymphonyUser["jabber_user_name"];
		$jabberPassword = $iSymphonyUser["jabber_password"];
	
	} else {
		$added = true;
		$userProfile = true;
		$password = "secret";
		$email = "";
		$cell_phone = "";
		$autoAnswer = false;	
		$jabberHost = "";
		$jabberDomain = "";
		$jabberResource = "iSymphony";
		$jabberPort = "5222";
		$jabberUserName = "";
		$jabberPassword = "";
	}
	
	//Create GIU elements if not on delete page
	if ($action != "del") {
		$section = _("iSymphony Profile Settings");
		$currentcomponent->addguielem($section,	new gui_checkbox("isymphony_add_profile", $userProfile, "Create Profile", "Creates an iSymphony login profile which is associated with this extension.", "on", "", "", false));
		$currentcomponent->addguielem($section, new gui_textbox("isymphony_profile_password", $password, "Profile Password", "Specifies the password to be used for this profile.", "", "", true, "100", false));	
		
		$section = _("iSymphony Extension Settings");
		$currentcomponent->addguielem($section,	new gui_checkbox("isymphony_add_extension", $added, "Add to iSymphony", "Makes this Extension/User available in iSymphony.", "on", "", "", false));
		$currentcomponent->addguielem($section,	new gui_checkbox("isymphony_auto_answer", $autoAnswer, "Auto Answer", "Makes this extension automatically answer the initial call received from the system when performing an origination within the panel. Only works with Aastra, Grandstream, Linksys, Polycom, and Snom phones.", "on", "", "", false));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_add_email", $email, "Email Address", "The email address entered here will be used whenever someone clicks the email icon for this extension.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_add_cell_phone", $cell_phone, "Cell Phone", "The number entered here will be used whenever someone clicks the cell phone icon for this extension.", "", "", true, "100"));

		$section = _("iSymphony Jabber Settings");
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_host", $jabberHost, "Host", "Host or IP of jabber server this extension will connect to.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_domain", $jabberDomain, "Domain", "Domain this extension will connect to the jabber server with.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_resource", $jabberResource, "Resource", "Resource this extension will connect to the jabber server with.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_port", $jabberPort, "Port", "Port this extension will connect to the jabber server with.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_user_name", $jabberUserName, "UserName", "UserName this extension will connect to the jabber server with.", "", "", true, "100"));
		$currentcomponent->addguielem($section,	new gui_textbox("isymphony_jabber_password", $jabberPassword, "Password", "Password this extension will connect to the jabber server with.", "", "", true, "100"));
	}
}

function isymphony_extension_configprocess() {
	
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$ext = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	$extn = isset($_REQUEST["extension"]) ? $_REQUEST["extension"]: null;
	$name = isset($_REQUEST["name"]) ? $_REQUEST["name"] : null;
	$extension = ($ext === "") ? $extn : $ext;
	
	//Determine peer
	if(isset($_REQUEST["devinfo_dial"]) && ($_REQUEST["devinfo_dial"] != "")) {
		$peer = $_REQUEST["devinfo_dial"];
	} else if (isset($_REQUEST["tech"])){
		$peer = strtoupper($_REQUEST["tech"]) . "/" . $extension;
	} else {
		$peer = "SIP/$extension";
	}
		
	//Query iSymphony options
	$addExtension = isset($_REQUEST["isymphony_add_extension"]);
	$addProfile = isset($_REQUEST["isymphony_add_profile"]);
	$autoAnswer = isset($_REQUEST["isymphony_auto_answer"]);
	$password = isset($_REQUEST["isymphony_profile_password"]) ? $_REQUEST["isymphony_profile_password"]: null;
	$password = (($password === null) || (trim($password) == "")) ? "secret" : trim($password);

	// Cell Phone
	$cell_phone = isset($_REQUEST["isymphony_add_cell_phone"]) ? $_REQUEST["isymphony_add_cell_phone"] : null;		
	
	// Email 
	$email = isset($_REQUEST["isymphony_add_email"]) ? $_REQUEST["isymphony_add_email"] : null;	
	
	//Jabber info
	$jabberHost = isset($_REQUEST["isymphony_jabber_host"]) ? $_REQUEST["isymphony_jabber_host"] : null;	
	$jabberDomain = isset($_REQUEST["isymphony_jabber_domain"]) ? $_REQUEST["isymphony_jabber_domain"] : null;	
	$jabberResource = isset($_REQUEST["isymphony_jabber_resource"]) ? $_REQUEST["isymphony_jabber_resource"] : "iSymphony";	
	$jabberPort = (isset($_REQUEST["isymphony_jabber_port"]) && is_numeric($_REQUEST["isymphony_jabber_port"])) ? $_REQUEST["isymphony_jabber_port"] : "5222";	
	$jabberUserName = isset($_REQUEST["isymphony_jabber_user_name"]) ? $_REQUEST["isymphony_jabber_user_name"] : null;	
	$jabberPassword = isset($_REQUEST["isymphony_jabber_password"]) ? $_REQUEST["isymphony_jabber_password"] : null;	

	//Modify DB
	if(($extension !== null) && ($extension != "") && ($action !== null)) {
		
		//Check if this extension needs to be deleted, updated, or added
		if($action == "del") {
			isymphony_user_del($extension);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_user_get($extension) === null) {
				isymphony_user_add($extension, $addExtension, $addProfile, $password, $name, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword);
			} else {
				isymphony_user_update($extension, $addExtension, $addProfile, $password, $name, $peer, $cell_phone, $email, $autoAnswer, $jabberHost, $jabberDomain, $jabberResource, $jabberPort, $jabberUserName, $jabberPassword);
			}
		}
	}
}

//Queue page hooks-----------------------------------------------------------------------------------------------------------------------------------
function isymphony_hook_queues($viewing_itemid, $target_menuid) {
		
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = "";
	
	//Only hook queues page
  	if(($target_menuid == "queues") && ($action != "delete")) {
  		
  		//Query queue info
  		if(($viewing_itemid != null) && ($iSymphonyQueue = isymphony_queue_get($viewing_itemid))) {
  			$checked = ($iSymphonyQueue["add_queue"] == "1") ? "checked" : "";
  		} else {
  			$checked = "checked";
  		}
  		
  		//Build display
  		$display = "	<tr><td colspan=\"2\"><h5>iSymphony<hr></h5></td></tr>
						<tr>
							<td><a href=\"#\" class=\"info\">" . _("Add to iSymphony") . "<span>" . _("Makes this queue available in iSymphony") . "</span></a></td>
							<td><input type=\"checkbox\" name=\"isymphony_add_queue\" id=\"isymphony_add_queue\" value=\"on\" $checked/></td>
						</tr>";
  	}
 
  	return $display;
}

function isymphony_hookProcess_queues($viewing_itemid, $request) {

	//Query page state
	$queue = isset($request["extdisplay"]) ? $request["extdisplay"] : null;
	$account = isset($request["account"]) ? $request["account"] : null;
	$action = isset($request["action"]) ? $request["action"] : null;
	$name = isset($request["name"]) ? $request["name"] : null;
	$queue = ($queue == null) ? $account : $queue;
	
	//Query iSymphony option
	$addQueue = isset($request["isymphony_add_queue"]);
	
	//Update DB
	if(($queue != null) && ($queue != "") && ($action != null)) {
		
		//Check if this queue needs to be deleted, updated, or added
		if($action == "delete") {
			isymphony_queue_del($queue);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_queue_get($queue) === null) {
				isymphony_queue_add($queue, $addQueue, $name);
			} else {
				isymphony_queue_update($queue, $addQueue, $name);
			}
		}
	}
}

//Conference Room page hooks-------------------------------------------------------------------------------------------------------------------------
function isymphony_hook_conferences($viewing_itemid, $target_menuid) {
		
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = "";
	
	//Only hook conferences page
  	if(($target_menuid == "conferences") && ($action != "delete")) {
  		
  		//Query conference info
  		if(($viewing_itemid != null) && ($iSymphonyConferenceRoom = isymphony_conference_room_get($viewing_itemid))) {
  			$checked = ($iSymphonyConferenceRoom["add_conference_room"] == "1") ? "checked" : "";
  		} else {
  			$checked = "checked";
  		}
  		
  		//Build display
  		$display = "	<tr><td colspan=\"2\"><h5>iSymphony<hr></h5></td></tr>
						<tr>
							<td><a href=\"#\" class=\"info\">" . _("Add to iSymphony") . "<span>" . _("Makes this conference room available in iSymphony") . "</span></a></td>
							<td><input type=\"checkbox\" name=\"isymphony_add_conference_room\" id=\"isymphony_add_conference_room\" value=\"on\" $checked/></td>
						</tr>";
  	}
 
  	return $display;
}

function isymphony_hookProcess_conferences($viewing_itemid, $request) {

	//Query page state
	$conferenceRoom = isset($request["extdisplay"]) ? $request["extdisplay"] : null;
	$account = isset($request["account"]) ? $request["account"] : null;
	$action = isset($request["action"]) ? $request["action"] : null;
	$name = isset($request["name"]) ? $request["name"] : null;
	$conferenceRoom = ($conferenceRoom == null) ? $account : $conferenceRoom;
	
	//Query iSymphony option
	$addConferenceRoom = isset($request["isymphony_add_conference_room"]);
	
	//Update DB
	if(($conferenceRoom != null) && ($conferenceRoom != "") && ($action != null)) {
		
		//Check if this conference room needs to be deleted, updated, or added
		if($action == "delete") {
			isymphony_conference_room_del($conferenceRoom);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_conference_room_get($conferenceRoom) === null) {
				isymphony_conference_room_add($conferenceRoom, $addConferenceRoom, $name);
			} else {
				isymphony_conference_room_update($conferenceRoom, $addConferenceRoom, $name);
			}
		}
	}
}

//Update checks--------------------------------------------------------------------------------------------------------------------------------------
function isymphony_location_update_check($serverRevision, $databaseValues, $maskedAutoReloadFlag, $maskedLogJabberMessagesFlag, $object) {
	
	//Checks for revision 1736 and above
	if($serverRevision >= 1736) {
			return (($object->asterisk_host != "localhost") || 
			($object->asterisk_port != "5038") || 
			($object->asterisk_login != "isymphony") || 
			($object->asterisk_password != "ismanager*con") || 
			($object->originate_timeout != $databaseValues["originate_timeout"]) ||
			($object->reload_on_dial_plan_reload != $maskedAutoReloadFlag) ||
			($object->jabber_host != $databaseValues["jabber_host"]) ||
			($object->jabber_domain != $databaseValues["jabber_domain"]) ||
			($object->jabber_resource != $databaseValues["jabber_resource"]) ||
			($object->jabber_port != $databaseValues["jabber_port"]) ||
			($object->device_user_mode != "true") ||
			($object->log_jabber_messages != $maskedLogJabberMessagesFlag));
	
	//Checks for revision 1489-1735
	} else if($serverRevision >= 1489) {
		return (($object->asterisk_host != "localhost") || 
				($object->asterisk_port != "5038") || 
				($object->asterisk_login != "isymphony") || 
				($object->asterisk_password != "ismanager*con") || 
				($object->originate_timeout != $databaseValues["originate_timeout"]) ||
				($object->reload_on_dial_plan_reload != $maskedAutoReloadFlag) ||
				($object->jabber_host != $databaseValues["jabber_host"]) ||
				($object->jabber_domain != $databaseValues["jabber_domain"]) ||
				($object->jabber_resource != $databaseValues["jabber_resource"]) ||
				($object->jabber_port != $databaseValues["jabber_port"]) ||
				($object->device_user_mode != "true"));
				
	//Checks for 1105-1488			
	} else if($serverRevision > 1104) {
		return (($object->asterisk_host != "localhost") || 
				($object->asterisk_port != "5038") || 
				($object->asterisk_login != "isymphony") || 
				($object->asterisk_password != "ismanager*con") || 
				($object->originate_timeout != $databaseValues["originate_timeout"]) ||
				($object->jabber_host != $databaseValues["jabber_host"]) ||
				($object->jabber_port != $databaseValues["jabber_port"]) ||
				($object->device_user_mode != "true"));
	
	//Checks for rev 1104 and below
	} else {
		
		return	(($object->asterisk_host != "localhost") || 
				($object->asterisk_port != "5038") || 
				($object->asterisk_login != "isymphony") || 
				($object->asterisk_password != "ismanager*con") || 
				($object->originate_timeout != $databaseValues["originate_timeout"]) ||
				($object->jabber_host != $databaseValues["jabber_host"]) ||
				($object->jabber_port != $databaseValues["jabber_port"]));
	}	
}

function isymphony_tenant_update_check($serverRevision, $databaseValues, $maskedPagedStatusEnabledFlag, $agentLoginContext, $object) {
	
	if($serverRevision >= 1660) {
			return (($object->originating_context != "from-internal") || 
					($object->redirecting_context != "from-internal") ||
					($object->agent_login_context != $agentLoginContext) ||
					($object->page_status_enabled != $maskedPagedStatusEnabledFlag) ||
					($object->page_context != "ext-paging") ||
					($object->record_file_name != "%EXT%-%DATE_TIME_FORMAT[yyyyMMdd-HHmmss]%-%ID%-%LINKED_ID%"));
	
	//Checks for 1520-1659
	} else if($serverRevision >= 1520) {
			return (($object->originating_context != "from-internal") || 
				($object->redirecting_context != "from-internal") ||
				($object->page_status_enabled != $maskedPagedStatusEnabledFlag) ||
				($object->page_context != "ext-paging") ||
				($object->record_file_name != "%EXT%-%DATE_TIME_FORMAT[yyyyMMdd-HHmmss]%-%ID%-%LINKED_ID%"));

	//Check for 1105-1519
	} else if($serverRevision > 1104) {
		return (($object->originating_context != "from-internal") || 
				($object->redirecting_context != "from-internal") ||
				($object->page_status_enabled != $maskedPagedStatusEnabledFlag) ||
				($object->page_context != "ext-paging"));
		
	//Checks for 1104 and below	
	} else {
		return ($object->originating_context != "from-internal") || ($object->redirecting_context != "from-internal");
	}														
}

function isymphony_extension_update_check($serverRevision, $databaseValues, $maskedAutoAnswerFlag, $agentLoginInterface, $object) {
	
	//Checks for 2203
	if($serverRevision >= 2203) {
		return (($object->cell_phone != $databaseValues["cell_phone"]) || 
				($object->email != $databaseValues["email"]) || 
				($object->name != $databaseValues["display_name"]) || 
				($object->voice_mail != $databaseValues["user_id"]) || 
				($object->peer != $databaseValues["peer"]) ||
				($object->auto_answer != $maskedAutoAnswerFlag) ||
				($object->agent_login_interface != $agentLoginInterface) ||
				($object->agent_login_name != $databaseValues["display_name"]));

	//Checks for 1957-2202
	} else if($serverRevision >= 1957) {
		return (($object->cell_phone != $databaseValues["cell_phone"]) || 
				($object->email != $databaseValues["email"]) || 
				($object->name != $databaseValues["display_name"]) || 
				($object->voice_mail != $databaseValues["user_id"]) || 
				($object->peer != $databaseValues["peer"]) ||
				($object->auto_answer != $maskedAutoAnswerFlag) ||
				($object->agent_login_interface != $agentLoginInterface));

	//Checks for 1493-1956
	} else if($serverRevision >= 1493) {
		return (($object->cell_phone != $databaseValues["cell_phone"]) || 
				($object->email != $databaseValues["email"]) || 
				($object->name != $databaseValues["display_name"]) || 
				($object->voice_mail != $databaseValues["user_id"]) || 
				($object->peer != $databaseValues["peer"]) ||
				($object->auto_answer != $maskedAutoAnswerFlag));
				
	//Checks for 1492 and below	
	} else {
		return (($object->cell_phone != $databaseValues["cell_phone"]) || 
				($object->email != $databaseValues["email"]) || 
				($object->name != $databaseValues["display_name"]) || 
				($object->voice_mail != $databaseValues["user_id"]) || 
				($object->peer != $databaseValues["peer"]));
	}
}

function isymphony_profile_update_check($serverRevision, $databaseValues, $maskedJabberPort, $object) {
	
	//Checks for revision 1489 and up
	if($serverRevision >= 1489) {
		return (($object->password != $databaseValues["password"]) ||         
			   	($object->jabber_host != $databaseValues["jabber_host"]) || 
				($object->jabber_domain != $databaseValues["jabber_domain"]) ||     
				($object->jabber_resource != $databaseValues["jabber_resource"]) ||  
				($maskedJabberPort != $databaseValues["jabber_port"]) || 
				($object->jabber_user_name != $databaseValues["jabber_user_name"]) || 
				($object->jabber_password != $databaseValues["jabber_password"]));
				
	//Checks for revision 1488 and below	
	} else {
		return (($object->password != $databaseValues["password"]) ||         
				($object->jabber_user_name != $databaseValues["jabber_user_name"]) || 
				($object->jabber_password != $databaseValues["jabber_password"]));		
	}
}

//Helper functions-----------------------------------------------------------------------------------------------------------------------------------
function isymphony_write_to_file($file, $content) {
	if($file) {
		fwrite($file,$content);
	}
}

function isymphony_remove_array_item($array, $item) {
	return explode(',',str_replace($item.',','',(join(',',$array))));
}
?>