<?php
/*
 *Name         : isymphony.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : Dec 15, 2010
 *History      : 0.10
 *Purpose      : Contains all functions used to interact with the iSymphony configuration.
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

//Build include path
$inc_path = __FILE__;
$pos = strrpos($inc_path, "/");
$inc_path = substr($inc_path, 0, $pos) . "/";

//Include Configuration
include($inc_path . "/configuration.php");

//Include Libraries
include($inc_path . "/library/navigation.php");
include($inc_path . "/library/helper.php");
include($inc_path . "/library/classes.php");
include($inc_path . "/library/property_arrays.php");

//Global Variables
$SOCKET = null;
$ISERROR = "";

//Tracker variables for improved running time
$CURRENT_LOCATION = null;
$CURRENT_TENANT = null;
$CURRENT_PROFILE = null;
$CURRENT_GROUP = null;

//Connection methods-------------------------------------------------------------------------------
/*###############################################
 * Initializes connection to iSymphony server
 * Takes: nothing
 * Returns: true if successful or an error message if not
 */
function iSymphonyConnect() {
	
	global $SOCKET, $ISERROR, $HOST, $PORT, $TIMEOUT;
	
	//Initiate socket
	set_time_limit($TIMEOUT);
	
	if(($SOCKET = socket_create(AF_INET, SOCK_STREAM, 0)) && (socket_connect($SOCKET, $HOST, $PORT))) {		
		
		//Check version and revision number to modify property arrays if necessary
		if((($versionResult = getiSymphonyServerVersion()) !== false) && (($revisionResult = getiSymphonyServerRevision()) !== false)) {
			
			//Modify for rev 2257
			if($revisionResult >= 2257) {
				isymphony_modify_property_arrays_2_2_2257();
				
			//Modify for rev 2205 through 2256
			} else if($revisionResult >= 2205) {
				isymphony_modify_property_arrays_2_2_2205();
			
			//Modify for rev 2203 through 2204
			} else if($revisionResult >= 2203) {
				isymphony_modify_property_arrays_2_2_2203();	
				
			//Modify for rev 2174 through 2202
			} else if($revisionResult >= 2174) {
				isymphony_modify_property_arrays_2_2_2174();		
				
			//Modify for rev 2124 through 2173
			} else if($revisionResult >= 2124) {
				isymphony_modify_property_arrays_2_2_2124();			

			//Modify for rev 2096 through 2123
			} else if($revisionResult >= 2096) {
				isymphony_modify_property_arrays_2_2_2096();	
				
			//Modify for rev 2041 through 2095
			} else if($revisionResult >= 2041) {
				isymphony_modify_property_arrays_2_2_2041();		
				
			//Modify for rev 1957 through 2040
			} else if($revisionResult >= 1957) {
				isymphony_modify_property_arrays_2_2_1957();
			
			//Modify for rev 1736 through 1956
			} else if($revisionResult >= 1736) {
				isymphony_modify_property_arrays_2_1_1736();
			
			//Modify for rev 1760 through 1735
			} else if($revisionResult >= 1760) {
				isymphony_modify_property_arrays_2_1_1706();
				
			//Modify for rev 1678 through 1759
			} else if($revisionResult >= 1678) {
				isymphony_modify_property_arrays_2_1_1678();
				
			//Modify for rev 1660 through 1677
			} else if($revisionResult >= 1660) {
				isymphony_modify_property_arrays_2_1_1660();
		
			//Modify for rev 1493 through 1659
			} else if($revisionResult >= 1493) {
				isymphony_modify_property_arrays_2_1_1493();
			
			//Modify for rev 1489 through 1493
			} else if($revisionResult >= 1489) {
				isymphony_modify_property_arrays_2_1_1489();
				
			//Modify for rev 1107 through 1492	
			} else if($revisionResult >= 1107) {
				isymphony_modify_property_arrays_2_1();
			}
		}
		return true;
	} else {
		$ISERROR = "Could not connect to server: " . socket_strerror(socket_last_error());
		return false;
	}
}

/*###############################################
 * Disconnects from iSymphony server
 * Takes: nothing
 * Returns: nothing 
 */
function iSymphonyDisconnect() {
	
	global $SOCKET;
		
	if(($SOCKET !== null) && ($SOCKET)) {
		sendAndRecive("exit");
		socket_close($SOCKET);
	}
	
	isymphony_reset_property_arrays();
	update_tracker();
}

//List methods-------------------------------------------------------------------------------------
/*###############################################
 * Gets list of locations
 * Takes: nothing
 * Returns: array containing the names of the locations 
 */
function getISymphonyLocationList() {

	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
	
	//Get location list
	return checkAndSetErrorList("list locations");
}

/*###############################################
 * Gets list of tenants for a given location
 * Takes: location name
 * Returns: array containing the names of the tenants 
 */
function getISymphonyTenantList($location) {

	//Move to location mode
	if(!moveToLocation($location)) {
		return false;
	}
	
	//Get tenant list
	return checkAndSetErrorList("list tenants");
}


/*###############################################
 * Gets list of extensions for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the extensions 
 */
function getISymphonyExtensionList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get extension list
	return checkAndSetErrorList("list extensions");
}

/*###############################################
 * Gets list of profiles for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the profiles 
 */
function getISymphonyProfileList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get profile list
	return checkAndSetErrorList("list profiles");
}

/*###############################################
 * Gets list of queues for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the queues 
 */
function getISymphonyQueueList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get queue list
	return checkAndSetErrorList("list queues");
}

/*###############################################
 * Gets list of conference rooms for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the conference rooms 
 */
function getISymphonyConferenceRoomList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get conference room list
	return checkAndSetErrorList("list meetme");
}

/*###############################################
 * Gets list of statuses for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the statuses 
 */
function getISymphonyStatusList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get status list
	return checkAndSetErrorList("list statuses");
}

/*###############################################
 * Gets list of permission groups for a given tenant
 * Takes: location name, tenant name
 * Returns: array containing the names of the permission groups 
 */
function getISymphonyPermissionGroupList($location, $tenant) {

	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Get permission group list
	return checkAndSetErrorList("list permgroups");
}

/*###############################################
 * Gets list of remote extension permissions of a specified extension group
 * Takes: location name, tenant name, permission group name
 * Returns: array containing the names of the extensions included in the permission list  
 */
function getISymphonyPermissionGroupRemoteExtensionPermissionList($location, $tenant, $group) {

	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Get permission group remote extension permission list
	return checkAndSetErrorList("list remote");
}

/*###############################################
 * Gets list of queue permissions of a specified extension group
 * Takes: location name, tenant name, permission group name
 * Returns: array containing the names of the queues included in the permission list  
 */
function getISymphonyPermissionGroupQueuePermissionList($location, $tenant, $group) {

	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Get permission group queue permission list
	return checkAndSetErrorList("list queue");
}

/*###############################################
 * Gets list of conference room permissions of a specified extension group
 * Takes: location name, tenant name, permission group name
 * Returns: array containing the names of the conference rooms included in the permission list  
 */
function getISymphonyPermissionGroupConferenceRoomPermissionList($location, $tenant, $group) {

	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Get permission group conference room permission list
	return checkAndSetErrorList("list meetme");
}

/*###############################################
 * Gets list of remote extension permission overrides of a specified profile
 * Takes: location name, tenant name, profile name
 * Returns: array containing the names of the extensions included in the permission list  
 */
function getISymphonyOverrideRemoteExtensionPermissionList($location, $tenant, $profile) {

	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Get override remote extension permission list
	return checkAndSetErrorList("list remote");
}

/*###############################################
 * Gets list of queue permission overrides of a specified profile
 * Takes: location name, tenant name, profile name
 * Returns: array containing the names of the queues included in the permission list  
 */
function getISymphonyOverrideQueuePermissionList($location, $tenant, $profile) {

	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Get override queue permission list
	return checkAndSetErrorList("list queue");
}

/*###############################################
 * Gets list of conference room permission overrides of a specified profile
 * Takes: location name, tenant name, profile name
 * Returns: array containing the names of the conference rooms included in the permission list  
 */
function getISymphonyOverrideConferenceRoomPermissionList($location, $tenant, $profile) {

	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Get override conference room permission list
	return checkAndSetErrorList("list meetme");
}

/*###############################################
 * Gets list of profile's managed extensions
 * Takes: location name, tenant name, profile name
 * Returns: array containing the extensions managed by the specified profile  
 */
function getISymphonyProfileExtensionList($location, $tenant, $profile) {

	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Get extension list
	return checkAndSetErrorList("list extensions");
}

/*###############################################
 * Gets list of permission group members
 * Takes: location name, tenant name, permission group name
 * Returns: array containing the permission group's member's  
 */
function getISymphonyPermissionGroupMemberList($location, $tenant, $group) {

	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Get member list
	return checkAndSetErrorList("list members");
}

//Property class query methods---------------------------------------------------------------------
/*###############################################
 * Gets an object that represents the servers properties
 * Takes: nothing
 * Returns: an ISymphonyServer object or false if error occurred  
 */
function getISymphonyServer() {
 	
 	global $serverPropertyArray, $ISERROR;
 	
 	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyServer;
		
		//Set properties of object
		foreach($serverPropertyArray as $val) {
			
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
 }
 
/*###############################################
 * Gets an object that represents a location's properties
 * Takes: location name
 * Returns: an ISymphonyLocation object or false if error occurred  
 */
function getISymphonyLocation($location) {
 	
 	global $locationPropertyArray, $ISERROR;
 	
 	//Move to location mode
	if(!moveToLocation($location)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyLocation;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
		
		//Set properties of object
		foreach($locationPropertyArray as $val) {
			
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
 }
 
/*###############################################
 * Gets an object that represents a tenant's properties
 * Takes: location name, tenant name
 * Returns: an ISymphonyTenant object or false if error occurred  
 */
function getISymphonyTenant($location, $tenant) {
 	
 	global $tenantPropertyArray, $ISERROR;
 	
 	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyTenant;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
		
		//Set properties of object
		foreach($tenantPropertyArray as $val) {
			
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
 }
 
/*###############################################
 * Gets an object that represents an extensions's properties
 * Takes: location name, tenant name, extension name
 * Returns: an ISymphonyExtension object or false if error occurred  
 */
function getISymphonyExtension($location, $tenant, $extension) {
 	
 	global $extensionPropertyArray, $ISERROR;
 	
 	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyExtension;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_extension_val = $propertyArray["extension_val"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($extensionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
 }
 
/*###############################################
 * Gets an object that represents a profiles's properties
 * Takes: location name, tenant name, profile name
 * Returns: an ISymphonyProfile object or false if error occurred  
 */
function getISymphonyProfile($location, $tenant, $profile) {
 	
 	global $profilePropertyArray, $ISERROR;
 	
 	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyProfile;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($profilePropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents a queues's properties
 * Takes: location name, tenant name, queue name
 * Returns: an ISymphonyQueue object or false if error occurred  
 */
function getISymphonyQueue($location, $tenant, $queue) {
 	
 	global $queuePropertyArray, $ISERROR;
 	
 	//Move to queue mode
	if(!moveToQueue($location, $tenant, $queue)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyQueue;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($queuePropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}  

/*###############################################
 * Gets an object that represents a conference room's properties
 * Takes: location name, tenant name, room name
 * Returns: an ISymphonyConferenceRoom object or false if error occurred  
 */
function getISymphonyConferenceRoom($location, $tenant, $room) {
 	
 	global $conferenceRoomPropertyArray, $ISERROR;
 	
 	//Move to conference room mode
	if(!moveToConferenceRoom($location, $tenant, $room)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyConferenceRoom;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($conferenceRoomPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}  

/*###############################################
 * Gets an object that represents a permission groups's properties
 * Takes: location name, tenant name, group name
 * Returns: an ISymphonyPermissionGroup object or false if error occurred  
 */
function getISymphonyPermissionGroup($location, $tenant, $group) {
 	
 	global $permissionGroupPropertyArray, $ISERROR;
 	
 	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyPermissionGroup;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($permissionGroupPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}  

/*###############################################
 * Gets an object that represents a status's properties
 * Takes: location name, tenant name, status name
 * Returns: an ISymphonyStatus object or false if error occurred  
 */
function getISymphonyStatus($location, $tenant, $status) {
 	
 	global $statusPropertyArray, $ISERROR;
 	
 	//Move to status mode
	if(!moveToStatus($location, $tenant, $status)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyStatus;
		
		//Set mode attributes
		if(array_key_exists("name", $propertyArray)) {
			$object->mode_location = $location;
			$object->mode_tenant = $tenant;
			$object->original_name = $propertyArray["name"];
		} else {
			$ISERROR = "Error: mode property \"name\" was not provided by the server.";
			return false;
		}
				
		//Set properties of object
		foreach($statusPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}   

/*###############################################
 * Gets an object that represents the default local permissions
 * Takes: nothing
 * Returns: an ISymphonyDefaultLocalPermission object or false if error occurred  
 */
function getISymphonyDefaultLocalPermissions() {
 	
 	global $localPermissionPropertyArray, $ISERROR;
 	
 	//Move to default local permission mode
	if(!moveToDefaultLocalPermission()) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyDefaultLocalPermission;
						
		//Set properties of object
		foreach($localPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents the default remote permissions
 * Takes: nothing
 * Returns: an ISymphonyDefaultRemotePermission object or false if error occurred  
 */
function getISymphonyDefaultRemotePermissions() {
 	
 	global $remotePermissionPropertyArray, $ISERROR;
 	
	//Move to default remote permission mode
	if(!moveToDefaultRemotePermission()) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyDefaultRemotePermission;
						
		//Set properties of object
		foreach($remotePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents the default park permissions
 * Takes: nothing
 * Returns: an ISymphonyDefaultParkPermission object or false if error occurred  
 */
function getISymphonyDefaultParkPermissions() {
 	
 	global $parkPermissionPropertyArray, $ISERROR;
 	
	//Move to default park permission mode
	if(!moveToDefaultParkPermission()) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyDefaultParkPermission;
						
		//Set properties of object
		foreach($parkPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents the default queue permissions
 * Takes: nothing
 * Returns: an ISymphonyDefaultQueuePermission object or false if error occurred  
 */
function getISymphonyDefaultQueuePermissions() {
 	
 	global $queuePermissionPropertyArray, $ISERROR;
 	
	//Move to default queue permission mode
	if(!moveToDefaultQueuePermission()) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyDefaultQueuePermission;
						
		//Set properties of object
		foreach($queuePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}  

/*###############################################
 * Gets an object that represents the default conference room permissions
 * Takes: nothing
 * Returns: an ISymphonyDefaultConferenceRoomPermission object or false if error occurred  
 */
function getISymphonyDefaultConferenceRoomPermissions() {
 	
 	global $conferenceRoomPermissionPropertyArray, $ISERROR;
 	
	//Move to default conference room permission mode
	if(!moveToDefaultConferenceRoomPermission()) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyDefaultConferenceRoomPermission;
						
		//Set properties of object
		foreach($conferenceRoomPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents a group's local permissions
 * Takes: location name, tenant name, group name
 * Returns: an ISymphonyGroupLocalPermission object or false if error occurred  
 */
function getISymphonyGroupLocalPermissions($location, $tenant, $group) {
 	
 	global $localPermissionPropertyArray, $ISERROR;
 	
	//Move to local permission mode
	if(!moveToLocalGroupPermission($location, $tenant, $group)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyGroupLocalPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_group = $group;
			
		//Set properties of object
		foreach($localPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a group's park permissions
 * Takes: location name, tenant name, group name
 * Returns: an ISymphonyGroupParkPermission object or false if error occurred  
 */
function getISymphonyGroupParkPermissions($location, $tenant, $group) {
 	
 	global $parkPermissionPropertyArray, $ISERROR;
 	
	//Move to local permission mode
	if(!moveToParkGroupPermission($location, $tenant, $group)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyGroupParkPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_group = $group;
			
		//Set properties of object
		foreach($parkPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a group remote extension permission set
 * Takes: location name, tenant name, group name, extension name
 * Returns: an ISymphonyGroupRemotePermission object or false if error occurred  
 */
function getISymphonyGroupRemotePermissions($location, $tenant, $group, $extension) {
 	
 	global $remotePermissionPropertyArray, $ISERROR;
 	
	//Move to remote permission mode
	if(!moveToRemoteGroupPermission($location, $tenant, $group, $extension)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyGroupRemotePermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_group = $group;
		$object->mode_extension = $extension;
			
		//Set properties of object
		foreach($remotePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a group queue permission set
 * Takes: location name, tenant name, group name, queue name
 * Returns: an ISymphonyGroupQueuePermission object or false if error occurred  
 */
function getISymphonyGroupQueuePermissions($location, $tenant, $group, $queue) {
 	
 	global $queuePermissionPropertyArray, $ISERROR;
 	
	//Move to queue permission mode
	if(!moveToQueueGroupPermission($location, $tenant, $group, $queue)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyGroupQueuePermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_group = $group;
		$object->mode_queue = $queue;
			
		//Set properties of object
		foreach($queuePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents a group conference room permission set
 * Takes: location name, tenant name, group name, room name
 * Returns: an ISymphonyGroupConferenceRoomPermission object or false if error occurred  
 */
function getISymphonyGroupConferenceRoomPermissions($location, $tenant, $group, $room) {
 	
 	global $conferenceRoomPermissionPropertyArray, $ISERROR;
 	
	//Move to conference room permission mode
	if(!moveToConferenceRoomGroupPermission($location, $tenant, $group, $room)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyGroupConferenceRoomPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_group = $group;
		$object->mode_room = $room;
			
		//Set properties of object
		foreach($conferenceRoomPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}  

/*###############################################
 * Gets an object that represents a profiles's local override permissions
 * Takes: location name, tenant name, profile name
 * Returns: an ISymphonyOverrideLocalPermission object or false if error occurred  
 */
function getISymphonyOverrideLocalPermissions($location, $tenant, $profile) {
 	
 	global $localPermissionPropertyArray, $ISERROR;
 	
	//Move to local permission mode
	if(!moveToLocalOverridePermission($location, $tenant, $profile)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyOverrideLocalPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_profile = $profile;
			
		//Set properties of object
		foreach($localPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a profiles's park override permissions
 * Takes: location name, tenant name, profile name
 * Returns: an ISymphonyOverrideParkPermission object or false if error occurred  
 */
function getISymphonyOverrideParkPermissions($location, $tenant, $profile) {
 	
 	global $parkPermissionPropertyArray, $ISERROR;
 	
	//Move to park permission mode
	if(!moveToParkOverridePermission($location, $tenant, $profile)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyOverrideParkPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_profile = $profile;
			
		//Set properties of object
		foreach($parkPermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a profile's remote extension permission set
 * Takes: location name, tenant name, profile name, extension name
 * Returns: an ISymphonyOverrideRemotePermission object or false if error occurred  
 */
function getISymphonyOverrideRemotePermissions($location, $tenant, $profile, $extension) {
 	
 	global $remotePermissionPropertyArray, $ISERROR;
 	
	//Move to remote permission mode
	if(!moveToRemoteOverridePermission($location, $tenant, $profile, $extension)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyOverrideRemotePermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_profile = $profile;
		$object->mode_extension = $extension;
			
		//Set properties of object
		foreach($remotePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

/*###############################################
 * Gets an object that represents a profile's queue permission set
 * Takes: location name, tenant name, profile name, queue name
 * Returns: an ISymphonyOverrideQueuePermission object or false if error occurred  
 */
function getISymphonyOverrideQueuePermissions($location, $tenant, $profile, $queue) {
 	
 	global $queuePermissionPropertyArray, $ISERROR;
 	
	//Move to queue permission mode
	if(!moveToQueueOverridePermission($location, $tenant, $profile, $queue)) {
		return false;
	}
	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyOverrideQueuePermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_profile = $profile;
		$object->mode_queue = $queue;
			
		//Set properties of object
		foreach($queuePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
} 

/*###############################################
 * Gets an object that represents a profile's conference room permission set
 * Takes: location name, tenant name, profile name, conference room name
 * Returns: an ISymphonyOverrideQueuePermission object or false if error occurred  
 */
function getISymphonyOverrideConferenceRoomPermissions($location, $tenant, $profile, $room) {
 	
 	global $queuePermissionPropertyArray, $ISERROR;
 	
	//Move to conference room permission mode
	if(!moveToConferenceRoomOverridePermission($location, $tenant, $profile, $room)) {
		return false;
	}
 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("view")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyOverrideConferenceRoomPermission;
				
		//Set mode properties
		$object->mode_location = $location;
		$object->mode_tenant = $tenant;
		$object->mode_profile = $profile;
		$object->mode_room = $room;
			
		//Set properties of object
		foreach($queuePermissionPropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

//Remove methods-----------------------------------------------------------------------------------
/*###############################################
 * Removes a location
 * Takes: location name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyLocation($location) {
	
	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove location $location");
}

/*###############################################
 * Removes a tenant
 * Takes: location name, tenant name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyTenant($location, $tenant) {
	
	//Move to location mode
	if(!moveToLocation($location)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove tenant $tenant");
}

/*###############################################
 * Removes an extension
 * Takes: location name, tenant name, extension name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyExtension($location, $tenant, $extension) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove extension $extension");
}

/*###############################################
 * Removes a profile
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyProfile($location, $tenant, $profile) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove profile $profile");
}

/*###############################################
 * Removes a queue
 * Takes: location name, tenant name, queue name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyQueue($location, $tenant, $queue) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove queue $queue");
}

/*###############################################
 * Removes a conference room
 * Takes: location name, tenant name, room name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyConferenceRoom($location, $tenant, $room) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove meetme $room");
}

/*###############################################
 * Removes a status
 * Takes: location name, tenant name, status name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyStatus($location, $tenant, $status) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove status $status");
}

/*###############################################
 * Removes a permission group
 * Takes: location name, tenant name, group name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyPermissionGroup($location, $tenant, $group) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove permgroup $group");
}

/*###############################################
 * Removes a group remote permission
 * Takes: location name, tenant name, group name, extension name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyGroupRemotePermission($location, $tenant, $group, $extension) {
	
	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove remote $extension");
}

/*###############################################
 * Removes a group queue permission
 * Takes: location name, tenant name, group name, queue name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyGroupQueuePermission($location, $tenant, $group, $queue) {
	
	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove queue $queue");
}

/*###############################################
 * Removes a group conference room permission
 * Takes: location name, tenant name, group name, room name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyGroupConferenceRoomPermission($location, $tenant, $group, $room) {
	
	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove meetme $room");
}

/*###############################################
 * Removes a profile override remote permission
 * Takes: location name, tenant name, profile name, extension name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyOverrideRemotePermission($location, $tenant, $profile, $extension) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove remote $extension");
}

/*###############################################
 * Removes a profile override queue permission
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyOverrideQueuePermission($location, $tenant, $profile, $queue) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove queue $queue");
}

/*###############################################
 * Removes a profile override conference room permission
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or false if error occurred  
 */
function removeISymphonyOverrideConferenceRoomPermission($location, $tenant, $profile, $room) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Remove element
	return checkAndSetErrorNone("remove meetme $room");
}

//License methods-----------------------------------------------------------------------------------
/*###############################################
 * Activates a tenant license 
 * Takes: location name, tenant name, serial
 * Returns: true if successful or false if error occurred  
 */
function activateISymphonyLicense($location, $tenant, $serial) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Activate license
	return checkAndSetErrorNone("license activate $serial");
} 

/*###############################################
 * Gets a name associated with a license
 * Takes: location name, tenant name
 * Returns: the license name if successful or false if error occurred  
 */
function getISymphonyLicenseName($location, $tenant) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Query license info
	return checkAndSetErrorString("license name");
} 

/*###############################################
 * Gets the number of clients of a licenses
 * Takes: location name, tenant name
 * Returns: the number of clients if successful or false if error occurred  
 */
function getISymphonyLicenseClients($location, $tenant) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Query license info
	return checkAndSetErrorString("license clients");
} 

/*###############################################
 * Gets the number of queues of a licenses
 * Takes: location name, tenant name
 * Returns: the number of queues if successful or false if error occurred  
 */
function getISymphonyLicenseQueues($location, $tenant) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Query license info
	return checkAndSetErrorString("license queues");
}

/*###############################################
 * Gets the number of trial days left of a licenses
 * Takes: location name, tenant name
 * Returns: the number of trial days left if successful or false if error occurred  
 */
function getISymphonyLicenseTrialDays($location, $tenant) {
	
	//Move to tenant mode
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Query license info
	return checkAndSetErrorString("license days");
}
 
//User status methods------------------------------------------------------------------------------
/*###############################################
 * Sets an extension's status
 * Takes: location name, tenant name, extension name, status
 * Returns: true if successful or false if error occurred  
 */ 
function setISymphonyExtensionStatus($location, $tenant, $extension, $status) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Set status
	return checkAndSetErrorNone("userstatus $status");
} 

/*###############################################
 * Sets an extension's status note
 * Takes: location name, tenant name, extension name, note
 * Returns: true if successful or false if error occurred  
 */ 
function setISymphonyExtensionNote($location, $tenant, $extension, $note) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Set note
	return checkAndSetErrorNone("note $note");	
} 

/*###############################################
 * Sets an extension's return time
 * Takes: location name, tenant name, extension name, time(unix time stamp)
 * Returns: true if successful or false if error occurred  
 */ 
function setISymphonyExtensionReturnTime($location, $tenant, $extension, $returntime) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Set return time
	return checkAndSetErrorNone("returntime $returntime");	
} 

/*###############################################
 * Gets an extension's status
 * Takes: location name, tenant name, extension name
 * Returns: the status name if successful or false if error occurred  
 */ 
function getISymphonyExtensionStatus($location, $tenant, $extension) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Get status
	return checkAndSetErrorString("query userstatus");
}

/*###############################################
 * Gets an extension's status note
 * Takes: location name, tenant name, extension name
 * Returns: the note if successful or false if error occurred  
 */ 
function getISymphonyExtensionNote($location, $tenant, $extension) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Get note
	return checkAndSetErrorString("query note");
}

/*###############################################
 * Gets an extension's returnTime
 * Takes: location name, tenant name, extension name
 * Returns: the return time as a unix time stamp if successful or false if error occurred  
 */ 
function getISymphonyExtensionReturnTime($location, $tenant, $extension) {
	
	//Move to extension mode
	if(!moveToExtension($location, $tenant, $extension)) {
		return false;
	}
	
	//Get return time
	return checkAndSetErrorString("query returntime");
}

//Profile managed extension methods----------------------------------------------------------------
/*###############################################
 * Adds an extension to a profile's managed list
 * Takes: location name, tenant name, profile name, extension name
 * Returns: true if successful or false if error occurred  
 */ 
function addISymphonyProfileManagedExtension($location, $tenant, $profile, $extension) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Add managed extension
	return checkAndSetErrorNone("add extension $extension");
}

/*###############################################
 * Removes an extension from a profile's managed list
 * Takes: location name, tenant name, profile name, extension name
 * Returns: true if successful or false if error occurred  
 */ 
function removeISymphonyProfileManagedExtension($location, $tenant, $profile, $extension) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Add managed extension
	return checkAndSetErrorNone("remove extension $extension");
}

//Permission group member methods------------------------------------------------------------------
/*###############################################
 * Adds a profile to a permission group's member list
 * Takes: location name, tenant name, permission group name, profile name
 * Returns: true if successful or false if error occurred  
 */ 
function addISymphonyPermissionGroupMember($location, $tenant, $group, $profile) {
	
	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Add member
	return checkAndSetErrorNone("add member $profile");
}

/*###############################################
 * Removes a profile from a permission group's member list
 * Takes: location name, tenant name, permission group name, profile name
 * Returns: true if successful or false if error occurred  
 */ 
function removeISymphonyPermissionGroupMember($location, $tenant, $group, $profile) {
	
	//Move to permission group mode
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Remove member
	return checkAndSetErrorNone("remove member $profile");
}

//Server methods-----------------------------------------------------------------------------------
/*###############################################
 * Gets the server version
 * Takes: nothing
 * Returns: server version if successful or false if error occurred  
 */ 
function getiSymphonyServerVersion() {
	
	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
	
	//Get server version
	return checkAndSetErrorString("version");
}

/*###############################################
 * Gets the server revision
 * Takes: nothing
 * Returns: server revision if successful or false if error occurred  
 */ 
function getiSymphonyServerRevision() {
	
	if(($serverVersion = getiSymphonyServerVersion()) === false) {
		return false;
	}
	
	//Parse our server revision number from server version
	$serverRevison = ereg_replace("[^0-9]+", '', substr($serverVersion, strpos($serverVersion, "rev")));
	
	//If revision number is not numeric something went wrong in the parsing. If so return false.
	if(!is_numeric($serverRevison)) {
		$ISERROR = "Error: Parsed server version was non numeric.";
		return false;
	}
	
	return $serverRevison;
}

/*###############################################
 * Shutdown the iSymphony server
 * Takes: nothing
 * Returns: nothing
 */ 
function shutdownISymphonyServer() {
	
	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
	
	//Shutdown server
	sendAndRecive("shutdown");
}

/*###############################################
 * Reload the iSymphony server
 * Takes: nothing
 * Returns: nothing
 */ 
function reloadISymphonyServer() {
	
	//Move to server mode
	if(!moveToServer()) {
		return false;
	}
	
	//Reload server
	sendAndRecive("reload");
}

/*###############################################
 * Reload a location
 * Takes: location name
 * Returns: true if successful or false if error occurred 
 */ 
function reloadISymphonyLocation($location) {
	
	//Move to location mode
	if(!moveToLocation($location)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorNone("reload");
}

/*###############################################
 * Gets a locations Asterisk connection status
 * Takes: location name
 * Returns: connection status if successful or false if error occurred  
 */ 
function getiSymphonyLocationConnectionStatus($location) {
	
	//Move to location mode
	if(!moveToLocation($location)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorString("connection");
}

/*###############################################
 * Checks for avaliable software updates
 * Takes: nothing
 * Returns: "Update Available: <version>" if an update is available, 
 * 			"No Update Available" if there is no update available, and 
 * 			false if an error occurred   
 */ 
function checkForiSymphonyUpdates() {
	
	//Check for updates
	return checkAndSetErrorString("check_for_updates");
}

/*###############################################
 * Perform software updates
 * Takes: nothing
 * Returns: nothing
 */ 
function iSymphonyUpdate() {
	
	//Perform updates
	return checkAndSetErrorString("update");
}

/*###############################################
 * Cancel currently running update
 * Takes: nothing
 * Returns: nothing
 */ 
function iSymphonyCancelUpdate() {
	
	//Cancel update
	return checkAndSetErrorString("cancel_update");
}

/*###############################################
 * Get current update state
 * Takes: nothing
 * Returns: an instance of ISymphonyUpdateState which describes the current state of an update or false if an error occurred. See ISymphonyUpdateState class definition for more detail.  
 */ 
function iSymphonyGetUpdateState() {
	
 	global $updateStatePropertyArray, $ISERROR;
 	 	
 	//Get list of properties
	if(($propertyArray = checkAndSetErrorList("update_state")) !== false) {
		
		//Convert return array to associative array
		$propertyArray = convertPropertyArray($propertyArray);

		//Create object from properties
		$object = new ISymphonyUpdateState;
				
		//Set properties of object
		foreach($updateStatePropertyArray as $val) {
						
			//Check if key exists in the associative array if it dose modify object if not set error and return false
			if(array_key_exists($val, $propertyArray)) {
				$object->$val = $propertyArray[$val];
			} else {
				$ISERROR = "Error: property \"$val\" was not provided by the server.";
				return false;
			}
		}		
	
		return $object;
	} else {
		return false;
	}
}

//Permission override activation methods-----------------------------------------------------------
/*###############################################
 * Activates a profile's local permission overrides
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or false if error occurred 
 */ 
function activateISymphonyLocalOverridePermissions($location, $tenant, $profile) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorNone("activate local");
}

/*###############################################
 * Deactivates a profile's local permission overrides
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or false if error occurred 
 */ 
function deactivateISymphonyLocalOverridePermissions($location, $tenant, $profile) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorNone("deactivate local");
}

/*###############################################
 * Activates a profile's park permission overrides
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or false if error occurred 
 */ 
function activateISymphonyParkOverridePermissions($location, $tenant, $profile) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorNone("activate park");
}

/*###############################################
 * Deactivates a profile's park permission overrides
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or false if error occurred 
 */ 
function deactivateISymphonyParkOverridePermissions($location, $tenant, $profile) {
	
	//Move to profile mode
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Reload location
	return checkAndSetErrorNone("deactivate park");
}
?>