<?php
/*
 *Name         : navigation.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : Dec 1, 2009
 *History      : 0.3  
 *Purpose      : Contains functions used to navigate through different modes of the CLI. 
 *Copyright    : 2008 HEHE Enterprises, LLC
 */


/*###############################################
 * Moves CLI to server mode
 * Takes: nothing
 * Returns: true if successful or false if not
 */
function moveToServer() {

	//Move to server
	$moved = checkAndSetErrorNone("server");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to location mode
 * Takes: location name
 * Returns: true if successful or false if not
 */
function moveToLocation($location) {
		
	//Move to location
	$moved = checkAndSetErrorNone("location $location");
	
	//Update tracker
	if($moved) {
		update_tracker($location);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to tenant mode
 * Takes: location name, tenant name
 * Returns: true if successful or false if not
 */
function moveToTenant($location, $tenant) {
	
	//Move to location
	if(!in_location($location)) {
		if(!moveToLocation($location)) {
			return false;
		}
	}
	
	//Move to tenant
	$moved = checkAndSetErrorNone("tenant $tenant");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to extension mode
 * Takes: location name, tenant name, extension name
 * Returns: true if successful or an error if not
 */
function moveToExtension($location, $tenant, $extension) {
	
	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to extension
	$moved = checkAndSetErrorNone("extension $extension");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to profile mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToProfile($location, $tenant, $profile) {

	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to profile
	$moved = checkAndSetErrorNone("profile $profile");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to queue mode
 * Takes: location name, tenant name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueue($location, $tenant, $queue) {

	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to queue
	$moved = checkAndSetErrorNone("queue $queue");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to conference room mode
 * Takes: location name, tenant name, room name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoom($location, $tenant, $room) {

	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to conference room
	$moved = checkAndSetErrorNone("meetme $room");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to status mode
 * Takes: location name, tenant name, status name
 * Returns: true if successful or an error if not
 */
function moveToStatus($location, $tenant, $status) {
	
	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to status
	$moved = checkAndSetErrorNone("status $status");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to permission group mode
 * Takes: location name, tenant name, group name
 * Returns: true if successful or an error if not
 */
function moveToPermissionGroup($location, $tenant, $group) {
	
	//Move to tenant
	if(!in_tenant($location, $tenant)) {
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
	}
	
	//Move to permission group
	$moved = checkAndSetErrorNone("permgroup $group");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $group);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to default local permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultLocalPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default local permissions
	$moved = checkAndSetErrorNone("permission local");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to default remote permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultRemotePermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default remote permissions
	$moved = checkAndSetErrorNone("permission remote");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to default queue permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultQueuePermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default queue permissions
	$moved = checkAndSetErrorNone("permission queue");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to default park permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultParkPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default park permissions
	$moved = checkAndSetErrorNone("permission park");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to default conference room permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultConferenceRoomPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default conference room permissions
	$moved = checkAndSetErrorNone("permission meetme");
	
	//Update tracker
	if($moved) {
		update_tracker();
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to local permission override mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToLocalOverridePermission($location, $tenant, $profile) {
	
	//Move to profile
	if(!in_profile($location, $tenant, $profile)) {
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
	}
	
	//Move to local permission override
	$moved =  checkAndSetErrorNone("permission local");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to park permission override mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToParkOverridePermission($location, $tenant, $profile) {
	
	//Move to profile
	if(!in_profile($location, $tenant, $profile)) {
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
	}
	
	//Move to park permission override
	$moved = checkAndSetErrorNone("permission park");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to remote permission override mode
 * Takes: location name, tenant name, profile name, extension name
 * Returns: true if successful or an error if not
 */
function moveToRemoteOverridePermission($location, $tenant, $profile, $extension) {
	
	//Move to profile
	if(!in_profile($location, $tenant, $profile)) {
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
	}
	
	//Move to remote permission override
	$moved = checkAndSetErrorNone("permission remote $extension");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to queue permission override mode
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueueOverridePermission($location, $tenant, $profile, $queue) {
	
	//Move to profile
	if(!in_profile($location, $tenant, $profile)) {
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
	}
	
	//Move to queue permission override
	$moved = checkAndSetErrorNone("permission queue $queue");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to conference room permission override mode
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoomOverridePermission($location, $tenant, $profile, $room) {
	
	//Move to profile
	if(!in_profile($location, $tenant, $profile)) {
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
	}
	
	//Move to conference room permission override
	$moved = checkAndSetErrorNone("permission meetme $room");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, $profile);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to local group permission mode
 * Takes: location name, tenant name, permission group name
 * Returns: true if successful or an error if not
 */
function moveToLocalGroupPermission($location, $tenant, $group) {
	
	//Move to permission group
	if(!in_group($location, $tenant, $group)) {
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
	}
	
	//Move to group local permission
	$moved = checkAndSetErrorNone("permission local");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, null, $group);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to park group permission mode
 * Takes: location name, tenant name, permission group name
 * Returns: true if successful or an error if not
 */
function moveToParkGroupPermission($location, $tenant, $group) {
	
	//Move to permission group
	if(!in_group($location, $tenant, $group)) {
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
	}
	
	//Move to group park permission
	$moved = checkAndSetErrorNone("permission park");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, null, $group);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to remote group permission mode
 * Takes: location name, tenant name, permission group name, extension name
 * Returns: true if successful or an error if not
 */
function moveToRemoteGroupPermission($location, $tenant, $group, $extension) {
	
	//Move to permission group
	if(!in_group($location, $tenant, $group)) {
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
	}
	
	//Move to group remote permission
	$moved = checkAndSetErrorNone("permission remote $extension");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, null, $group);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to queue group permission mode
 * Takes: location name, tenant name, permission group name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueueGroupPermission($location, $tenant, $group, $queue) {
	
	//Move to permission group
	if(!in_group($location, $tenant, $group)) {
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
	}
	
	//Move to group queue permission
	$moved = checkAndSetErrorNone("permission queue $queue");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, null, $group);
	}
	
	return $moved;
}

/*###############################################
 * Moves CLI to conference room group permission mode
 * Takes: location name, tenant name, permission group name, conference room name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoomGroupPermission($location, $tenant, $group, $room) {
	
	//Move to permission group
	if(!in_group($location, $tenant, $group)) {
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
	}
	
	//Move to group conference room permission
	$moved = checkAndSetErrorNone("permission meetme $room");
	
	//Update tracker
	if($moved) {
		update_tracker($location, $tenant, null, $group);
	}
	
	return $moved;
}
?>